@@ -211,12 +211,16 @@ class PXTester(EATester):
             #
             precision = item.get('precision', 2)
             item_type = item.get('type', 'value')
-            if item_type == 'value':
-                report_str += f"{idx:02d}). {item['desc']}: {round(item['value'], precision)}\n"
-            elif item_type == '%': #%
-                report_str += f"{idx:02d}). {item['desc']}: {round(item['value']*100, precision)} %\n"
-            else: #str
-                report_str += f"{idx:02d}). {item['desc']}: {item['value']}\n"
+            rs = None
+            if item['value'] is not None:
+                if item_type == 'value':
+                    rs = f"{idx:02d}). {item['desc']}: {round(item['value'], precision)}\n"
+                elif item_type == '%': #%
+                    rs = f"{idx:02d}). {item['desc']}: {round(item['value']*100, precision)} %\n"
+            #str
+            if rs is None:
+                rs = f"{idx:02d}). {item['desc']}: {item['value']}\n"
+            report_str += rs
 
             idx += 1
         self.write_log(f"{report_str}", type='report')
@@ -280,10 +284,14 @@ class PXTester(EATester):
             #                         ('l', float), ('v', float), ('a', float), ('b', float), ])
         return raw_data, data
 
-    def symbol_datagetitem_callback(self, data, data_index_ts, timeframe, timeframe_seconds, shift, fail_value):
+    def symbol_datagetitem_index(self, data_index_ts, timeframe, timeframe_seconds, shift):
         t = int(self.tick_info[self.current_tick_index]['t']/ timeframe_seconds)*timeframe_seconds
         cidx = np.where(data_index_ts == t)[0][0]
         idx = cidx - shift
+        return idx
+
+    def symbol_datagetitem_callback(self, data, data_index_ts, timeframe, timeframe_seconds, shift, fail_value):
+        idx = self.symbol_datagetitem_index(data_index_ts, timeframe, timeframe_seconds, shift)
         # idx = self.current_tick_index - shift
         if idx < 0:
             return fail_value
